/*
	File:		HYPluginInterface.h
	
	Description: iMovie plugin interface

	Author:		iMovie Engineering team

	Copyright: 	 Copyright 2003 Apple Computer, Inc. All rights reserved.
	
	Disclaimer:	IMPORTANT:  This Apple software is supplied to you by Apple Computer, Inc.
				("Apple") in consideration of your agreement to the following terms, and your
				use, installation, modification or redistribution of this Apple software
				constitutes acceptance of these terms.  If you do not agree with these terms,
				please do not use, install, modify or redistribute this Apple software.

				In consideration of your agreement to abide by the following terms, and subject
				to these terms, Apple grants you a personal, non-exclusive license, under Apples
				copyrights in this original Apple software (the "Apple Software"), to use,
				reproduce, modify and redistribute the Apple Software, with or without
				modifications, in source and/or binary forms; provided that if you redistribute
				the Apple Software in its entirety and without modifications, you must retain
				this notice and the following text and disclaimers in all such redistributions of
				the Apple Software.  Neither the name, trademarks, service marks or logos of
				Apple Computer, Inc. may be used to endorse or promote products derived from the
				Apple Software without specific prior written permission from Apple.  Except as
				expressly stated in this notice, no other rights or licenses, express or implied,
				are granted by Apple herein, including but not limited to any patent rights that
				may be infringed by your derivative works or by other works in which the Apple
				Software may be incorporated.

				The Apple Software is provided by Apple on an "AS IS" basis.  APPLE MAKES NO
				WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
				WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
				PURPOSE, REGARDING THE APPLE SOFTWARE OR ITS USE AND OPERATION ALONE OR IN
				COMBINATION WITH YOUR PRODUCTS.

				IN NO EVENT SHALL APPLE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR
				CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
				GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
				ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION
				OF THE APPLE SOFTWARE, HOWEVER CAUSED AND WHETHER UNDER THEORY OF CONTRACT, TORT
				(INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN IF APPLE HAS BEEN
				ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
				
	Change History (most recent first):

*/

#ifndef __HYPluginInterface__
#define __HYPluginInterface__

#import <Carbon/Carbon.h>

#ifdef __cplusplus
extern "C" {
#endif

#include "HYPluginVersion.h"

// constants and definitions specific to plug-in architecture

#define kPluginFileCreator				'Hway'
#define kPluginTitleFileType			'titl'
#define kPluginTransitionFileType		'tran'
#define kPluginFilterFileType			'fltr'
#define kResProgrammerNames				15000
#define kResMenuNames					16000
#define kResGroupNames					18000

// Note: Do NOT add any English STR# resources that are between 16000 and 16999
// This range of IDs is reserved for localized versions of kResMenuNames

// Note: Do NOT add any English STR# resources that are between 17000 and 17999
// This range of IDs is reserved for localized versions of Progress Strings

// Note: Do NOT add any English STR# resources that are between 18000 and 18999
// This range of IDs is reserved for localized versions of Group Strings

// -----------------

// to use Custom buttons in the effects panel, you must enable the plug-in flag
// kPluginChoiceFileButton which will enable/show the button.  When the user
// clicks the button, your plug-in will be called with the new selector
// kPlugCustomButtonClick.
	
#define PROC_DECL

// result codes
typedef enum _HYPluginResult {
	kPluginOK = 0,
	kPluginCancel,
	kPluginFinished,
	kPluginBadParams,
	kPluginOutOfMemory,
	kPluginOutOfDiskSpace,
	kPluginFailed,
	kPluginCantGetNeededFrame,
	kPluginUnimplemented,
	kPluginTryAgainLater,
	kPluginAppIsTooOld,
	kPluginAppIsTooNew
} HYPluginResult;

// selectors
typedef enum _HYPluginSelector {
	kPlugGetInfo = 1,
	kPlugInitialize,
	kPlugDrawIcon,
	kPlugDrawThumbNail,
	kPlugFilePlaceHolder,
	kPlugDoFrame,
	kPlugCallOnceWithInitAndTerminate,
	kPlugTerminate,
	// new for version 3:
	kPlugCustomButtonClick
} HYPluginSelector;

// constants and definitions specific to video

typedef void *HYFrameCookie;
typedef void *HYPluginInstance;

// HYRefDigitalVideoStandard
typedef enum _RefVideoStandards {
	kRefUnknownStandard = 0,
	kRefNTSCStandard = 1,
	kRefPALStandard = 2
} HYRefDigitalVideoStandard;

typedef enum _HYRefDVQuality {
	kDVLowQuality,
	kDVHighQuality,
	kDVHighQualitySingleField
} HYRefDVQuality;

// HYRefClipType
typedef enum {
	kRefUnknownClipType = 0,
	kRefRegularClipType,
	kRefTransitionClipType,
	kRefTitleClipType,
	// added in version 4.0 of the API:
	kRefImageClipType,
	kRefBlackClipType,
	kRefPanZoomClipType
} HYRefClipType;

// HYRefClip
typedef struct _HYRefClip {
	HYRefClipType clipType;
	char name[128];
	HYRefDigitalVideoStandard standard;
	long frameCount;
	long in;
	long out;
	FSSpec fileSpec;
	// added in version 4.0 of the API:
	long absoluteStartFrame;
} HYRefClip;

// HYPluginType
typedef enum {
	kPluginUnknownType = 0,
	kPluginTitleType = 101,
	kPluginTransitionBeforeType,
	kPluginTransitionBetweenType,
	kPluginTransitionAfterType,
	kPluginFilterType
} HYPluginType __attribute ((packed));

// HYTitleType
typedef enum {
	kTitleUnknown = 1,
	kTitleSingleTextPair,
	kTitleMultiTextPair,
	kTitleTextBlock
} HYTitleType __attribute ((packed));

typedef enum {					// based on NSTextAlignment, but avoids including NSText.h
    HYLeftTextAlignment		= 0, /* Visually left aligned */
    HYRightTextAlignment	= 1, /* Visually right aligned */
    HYCenterTextAlignment	= 2,
    HYJustifiedTextAlignment	= 3,
    HYNaturalTextAlignment	= 4  /* Indicates the default alignment for script */
} HYTextAlignment;

typedef struct _HYPluginTitlePair {
	char *title;
	char *subTitle;	
} HYPluginTitlePair;

typedef unsigned long HYPluginChoiceFlags;

// HYPluginChoiceFlags
#define kPluginChoiceVertical	  (1L)
#define kPluginChoiceHorizontal	  (2L)
#define kPluginChoiceSpeed		  (4L)

#define kPluginChoiceAll		  (7L)

#define PLUGIN_COMBO_VERTICAL (kPluginChoiceAll & ~kPluginChoiceHorizontal)
#define PLUGIN_COMBO_HORIZONTAL (kPluginChoiceAll & ~kPluginChoiceVertical)
#define PLUGIN_COMBO_NOMOTION (PLUGIN_COMBO_VERTICAL & ~kPluginChoiceVertical)

// Title specific flags
#define kTitleChoiceSize		  (8L)
#define kTitleChoiceFont		 (16L)
#define kTitleChoiceHangTime	 (32L)
#define kTitleChoiceNoScaling	 (64L)

#define kTitleChoiceAll			(127L)

// Filter specific flags
#define kFilterChoiceFadeIn		(128L)
#define kFilterChoiceFadeOut	(256L)
#define kFilterChoiceAnimated	(512L)			// effect is different over time

// some handy combinations of HYPluginChoiceFlags for Titles:
#define TITLE_COMBO_STANDARD (kPluginChoiceVertical | kPluginChoiceHorizontal | kPluginChoiceSpeed | kTitleChoiceFont )
#define TITLE_COMBO_VERTICAL (TITLE_COMBO_STANDARD & ~kPluginChoiceHorizontal)
#define TITLE_COMBO_HORIZONTAL (TITLE_COMBO_STANDARD & ~kPluginChoiceVertical)
#define TITLE_COMBO_NOMOTION (TITLE_COMBO_VERTICAL & ~kPluginChoiceVertical)

// new as of version 3 of the API
#define kPluginChoiceXYLocation		(1024L)
#define kPluginChoiceFileButton		(2048L)
#define kPluginChoiceCustomButton	(4096L)

#define MAX_FILTER_SLIDERS 	3
#define MAX_TITLE_SLIDERS 	1

typedef enum {
	kPluginDirectionNone,
	kPluginDirectionUp,
	kPluginDirectionDown,
	kPluginDirectionLeft,
	kPluginDirectionRight
} HYPluginDirection __attribute ((packed));				// simple choices for now

typedef enum {
	kTitleJustCenter			= 0,
	kTitleJustLeft				= 1,
	kTitleJustRight				= 2
} HYTitleJustification __attribute ((packed));

// new to Plugin Interface Version 2.0 -------------
typedef struct _HY2PluginClipInfo {
	Boolean backwards;
	short frameSpeed;
	long fadeInFrames;
	long fadeOutFrames;
} HY2PluginClipInfo;

typedef struct _HY2PluginTransitionInfo {
	HYPluginDirection direction;
} HY2PluginTransitionInfo;

typedef struct _HY2PluginTitleInfo {
	float 	fontSizeScale;	// 0.0->1.0 This value does't represent the actual scale factor just the
							// value of the TitleFontSize slider.  It is up to each plugin to determine
							// what this value actually in terms of actual font size
	float 	hangTime;		// 0.0->1.0
} HY2PluginTitleInfo ;//__attribute__ ((aligned (2)));

// new to Plugin Interface Version 4.0 -------------
typedef struct _HY4PluginTitleInfo {
	char	fontName[256];		// font name; this supersedes the "short fontID" in
								// the HYPluginTitleInfo field which is mostly useless on OS X
	float	sliderValues[MAX_TITLE_SLIDERS];
} HY4PluginTitleInfo;

typedef struct _HY4PluginClipInfo {
	long clipFrame;				// pass in frame number here to get clipInfo for clip containing frame
	HYRefClip clipRef;			// more clip info here than in previous versions
} HY4PluginClipInfo;


// -------------

// HYPluginTitleInfo
typedef struct _HYPluginTitleInfo {
	RGBColor foreColor;			// text color
	RGBColor shadowColor;		// shadow color
	Boolean overBlack;			// if over black
	Boolean tvSafe;				// if TV Safe margins (and colors?) wanted
	short fontID;				// Mac font ID (DEPRECATED)
	float speed;				// slow to fast, in range 0.0 - 1.0
	long duration;				// in frames
	HYPluginDirection direction;	// direction of motion for title
	// -- actual text data
	short pairCount;			// used for "Pair" style plugins
	HYPluginTitlePair *pairs;	// used for "Pair" style plugins

	long blockLen;				// used for "Block" style plugins
	char *textBlock;			// used for "Block" style plugins
	
	HY2PluginTitleInfo ver2;
	// no changes in version 3
	HY4PluginTitleInfo ver4;
} HYPluginTitleInfo;			//__attribute__ ((aligned (2)));

// HYPluginClipInfo
typedef struct _HYPluginClipInfo {
	short volume;
	Boolean fadeIn;
	Boolean fadeOut;
	char name[256];
	HY2PluginClipInfo ver2;
	// no changes in version 3
	HY4PluginClipInfo ver4;		// clipFrame in here, and expanded clipInfo
} HYPluginClipInfo;


// HYPluginTransitionInfo
typedef struct _HYPluginTransitionInfo {
	float speed;
	HY2PluginTransitionInfo ver2;
} HYPluginTransitionInfo;

typedef struct _HYPluginFilterSliderInfo {
	char label[128];		// if label[0] == '\0', this means that slider is disabled
	char minLabel[128];
	char maxLabel[128];
	float defaultValue;		// plugin specifies what the default value should be between 0.0 and 1.0.
} HYPluginFilterSliderInfo;

// HYPluginFilterInfo - new to Plugin Interface Version 2.0
typedef struct _HYPluginFilterInfo {
	float sliderValues[MAX_FILTER_SLIDERS];
	long fadeInFrames;
	long fadeOutFrames;
} HYPluginFilterInfo;

// new in version 3:
typedef struct _HYPluginFileOpenInfo {
	short				typeCount;
	Handle				hTypes;		// really a NavTypeListHandle but we don't include <Navigation.h>
	FSSpec				chosenFile;
} HYPluginFileOpenInfo;

typedef struct _HY2PluginGetInfo {
	char group[128];				// plugins can only have names up to 128 chars long
	// For Filters
	HYPluginFilterSliderInfo filterSliders[MAX_FILTER_SLIDERS];
	long defaultFadeInFrames;
	long defaultFadeOutFrames;
	// For Titles
	long minHangFrames;				// for end points of Pause sliders
	long maxHangFrames;				// for end points of Pause sliders
	float defaultFontSizeScale;		// 0.0->1.0 This value does't represent the actual scale factor just the
									// value of the TitleFontSize slider.  It is up to each plugin to determine
									// what this value actually in terms of actual font size
	long selectedFrames;			// used by Filters to know how many frames are selected
	long speedFrames;				// returned by plug-in to app based on params
	long hangFrames;				// returned by plug-in to app based on params
	long baseMemoryNeeded;			// how much mem allocated in "Begin" as overhead
} HY2PluginGetInfo;

typedef struct _HY3PluginGetInfo {
	// for XY location
	float defaultX;						// starting X,Y established by plug-in
	float defaultY;
	float userX;						// draggable X location
	float userY;						// draggable X location
	// for "File..." button
	HYPluginFileOpenInfo fileInfo;
	// for Custom button
	char buttonTitle[32];
	
} HY3PluginGetInfo;

typedef struct _HY4PluginGetInfo {
	HYPluginFilterSliderInfo titleSliders[MAX_TITLE_SLIDERS];	
} HY4PluginGetInfo;

// HYPluginGetInfo
typedef struct _HYPluginGetInfo {
	HYPluginType pluginType;			// transition, title
	HYTitleType titleType;				// single-pair, multi-pair, block
	HYPluginChoiceFlags pluginFlags;	// speed, direction, font, color..
	long framesToBeRendered;			// returned by plug-in to app based on params
	long takeFromClipBefore;			// how many frames to be taken for rendering
	long takeFromClipAfter;				// how many frames to be taken for rendering
	long minSpeedFrames;				// for end points of Speed sliders
	long maxSpeedFrames;				// for end points of Speed sliders
	Boolean mustTakeFromSingleClips;	// don't allow plug-in to consume frames from multiple clips
	long bestThumbOffset;				// which frame would make the best thumbnail
	RGBColor defaultColor;				// defaults for when no choices are supported
	HYPluginDirection defaultDirection;	// default direction
	HYTitleJustification justify;		// justification for current direction settings
	short defaultFontNum;
	short defaultFontSize;				// This field is not used
	// don't add fields here because it's in-line in the original HYPluginParams struct
	// and can't change size without breaking old plugins
} HYPluginGetInfo;

// -------------------------------------------------

#include "HYPluginProcs.h"

// -------------------------------------------------

// HYPluginParams
typedef struct _PluginParams {
	short interfaceVersion;			// for versioning control over plugins
	HYPluginInstance instance;		// uniquely identify the instance of the plugin
	HYPluginSelector selector;
	HYProcs *procs;
	// video-specific params
	HYPluginGetInfo getInfo;		// supply info back to app
	long startFrame;				// first effect frame (global frame numbers)
	long renderFrame;				// current frame (local, 0-based numbers)
	long frameCount;				// how many frames to be rendered by plugin
	Boolean proxyOnly;				// used for previews, etc, to avoid GetFrame calls
	Rect frameRect;					// within GWorld (not necessarily 0,0, could be window)
	GWorldPtr frame;				// a GWorldPtr on the Mac
	void *clientData;				// plugin can hook stuff on here
	
	HY2PluginGetInfo getInfo2;		// version 2 of the plug-in interface: extensions
	HY3PluginGetInfo getInfo3;		// version 3 of the plug-in interface: extensions
	HY4PluginGetInfo getInfo4;		// version 4 of the plug-in interface: extensions
} HYPluginParams;

// HYPluginMainProcPtr
typedef HYPluginResult (*HYPluginMainProcPtr) ( short which, HYPluginParams *params );

#ifdef __cplusplus
}
#endif


#endif
// __HYPluginInterface__

